# -*- coding: utf-8 -*-

#   Copyright 2010-2014 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module defining unarmed attack related objcts

Classes:
UnarmedToHit
UnarmedDamage
"""

from pyherc.aspects import log_debug
from pyherc.rules.combat.action import ToHit


class UnarmedToHit(ToHit):
    """
    Class to perform to hit calculations in unarmed combat
    """
    @log_debug
    def __init__(self, attacker, target, random_number_generator):
        """
        Default constructor

        Args:
            attacker: Character doing the attack
            target: Character being attacked
            random_number_generator: Random number generator
        """
        super().__init__(attacker, target, random_number_generator)
